/* eslint-disable global-require */

const { declare } = require('@babel/helper-plugin-utils');

module.exports = declare((api, options) => {
    // see docs about api at https://babeljs.io/docs/en/config-files#apicache
    api.assertVersion('^7.0.0');

    const {
        modules = 'auto',
        useBuiltIns = false,
        corejs = 2,
        looseClasses = false,
        runtimeVersion,
        runtimeHelpersUseESModules = !modules,
        transformRuntime = true
    } = options;

    if (typeof modules !== 'undefined' && typeof modules !== 'boolean' && modules !== 'auto') {
        throw new TypeError(
            '@justia/babel-preset only accepts `true`, `false`, or `"auto"` as the value of the "modules" option'
        );
    }

    const debug = typeof options.debug === 'boolean' ? options.debug : false;
    const development =
        typeof options.development === 'boolean'
            ? options.development
            : api.cache.using(() => process.env.NODE_ENV === 'development');

    return {
        presets: [
            [
                require('@babel/preset-env'),
                {
                    debug,
                    exclude: ['transform-async-to-generator', 'transform-template-literals', 'transform-regenerator'],
                    modules: modules === false ? false : 'auto',
                    corejs,
                    useBuiltIns,
                    shippedProposals: options.shippedProposals === true
                }
            ],
            [require('@babel/preset-react'), { development }]
        ],
        plugins: [
            looseClasses
                ? [
                      require('@babel/plugin-transform-classes'),
                      {
                          loose: true
                      }
                  ]
                : null,
            [
                require('@babel/plugin-transform-template-literals'),
                {
                    spec: true
                }
            ],
            require('@babel/plugin-transform-property-mutators'),
            require('@babel/plugin-transform-member-expression-literals'),
            require('@babel/plugin-transform-property-literals'),
            require('@babel/plugin-proposal-nullish-coalescing-operator'),
            require('@babel/plugin-proposal-optional-chaining'),
            [
                require('@babel/plugin-proposal-object-rest-spread'),
                {
                    useBuiltIns: true
                }
            ],
            transformRuntime
                ? [
                      require('@babel/plugin-transform-runtime'),
                      {
                          absoluteRuntime: false,
                          corejs: false,
                          helpers: true,
                          regenerator: false,
                          useESModules: runtimeHelpersUseESModules,
                          version: runtimeVersion
                      }
                  ]
                : null
        ].filter(Boolean)
    };
});
